#! /usr/bin/env python

class bg_generator:
  def __init__(self, numAgents, outputfile):
    self.numAgents = numAgents
    self.outputfile = outputfile    

  def formulaComposition(self, head, tails, isAnd):
    lenTails = len(tails)    
    if lenTails == 0:
      return '(' + head + ')'
    elif lenTails == 1:
      if isAnd:        
        #return '(' + head + '/\\' + tails[0] + ')'
        return head + ' /\\ ' + tails[0]
      else:
        #return '(' + head + '\\/' + tails[0] + ')'
        return head + ' \\/ ' + tails[0]
    else:
      fsttail = tails[0]
      if isAnd:
       newhead = head + ' /\\ ' + fsttail
      else:
       newhead = head + ' \\/ ' + fsttail
      return self.formulaComposition(newhead, tails[1:], isAnd)     

  def defDataType(self):
    'define the data type for dc protocol'
    if self.numAgents % 2 == 0:
     self.numMsg = self.numAgents / 2
     self.numAck = self.numMsg      
    else:
     self.numMsg = self.numAgents/2 + 1
     self.numAck = self.numAgents/2
    self.outputfile.write('''\
sndmsg : Bool[%d]\n
rcvmsg : Bool[%d]\n
sndack : Bool[%d]\n
rcvack : Bool[%d]\n''' % (self.numMsg, self.numMsg, self.numAck, self.numAck))
    self.outputfile.write( '\n' )
    self.outputfile.write('''\
msgReceived : Bool\n 
ackReceived : Bool\n\n''')  

  def generateAtomElem(self, atomStr):
    elems = []
    for i in range(self.numMsg):
      elems.append( (atomStr + 'sndmsg[%d]') % i )
      elems.append( (atomStr + 'rcvmsg[%d]') % i )
    for i in range(self.numAck):
      elems.append( (atomStr + 'sndack[%d]') % i )
      elems.append( (atomStr + 'rcvack[%d]') % i )
    return elems

  def defInitCond(self):
    'define the initial condition'
    # generate paid[0] ~ paid[n-1]
    paids = []
    elems = self.generateAtomElem('neg ')
    cond = self.formulaComposition(elems[0], elems[1:], True)    
     
    self.outputfile.write( 'init_cond = \n')  
    self.outputfile.write( cond + ' /\\ neg msgReceived /\\ neg ackReceived  ' )
    self.outputfile.write( '\n\n' )

  def defAgents(self):
    self.outputfile.write('''\
agent Alice "sender" (sndmsg, rcvack, ackReceived)
agent Bob  "receiver" (sndack, rcvmsg, msgReceived)''')
    self.outputfile.write( '\n\n' )      

  def generateMessageClause(self, msgStr, nums):
    elems = []
    if nums == 0:
      elems.append( ('neg ' + msgStr + '[%d]') % (nums) )
    else:
      elems.append( (msgStr + '[%d]') % (nums-1) )
      elems.append( ('neg ' + msgStr + '[%d]') % nums )
    elemslen = len(elems)
      
    if elemslen == 1:
      return elems[0] + ( (' -> ' + msgStr + '[%d] := True') % nums )
    else:
      clause = self.formulaComposition(elems[0], elems[1:], True)
      allclause = clause + ( (' -> ' + msgStr + '[%d] := True') % nums )
      return allclause

  def generateMessageClauses(self, ifClause, bracketClause, msgStr, numMsgs):
    for i in range(numMsgs):
      if i == 0:
        clause = ifClause + ' /\ ' + self.generateMessageClause(msgStr, i) + '\n'
      else:
        clause = bracketClause + ' /\ ' + self.generateMessageClause(msgStr, i) + '\n'
      self.outputfile.write(clause)
    self.outputfile.write('fi')

  def generateChannelMsgs(self, msgStr1, msgStr2, msgStr3, numMsgs):
    elems = []
    for i in range(numMsgs):
      elems.append( ('(' + msgStr1 + '[%d] /\\ neg ' + msgStr2 + '[%d]' + ')') % (i, i))
    if len(elems) == 1:
      clause = elems[0]
    else:
      clause = self.formulaComposition(elems[0], elems[1:], False)
    self.outputfile.write('if '+clause+'-> ' + msgStr3 + ' := True\n')
    self.outputfile.write('[] True -> ' + msgStr3 + ' := False\n')
    self.outputfile.write('fi')

  def defTransaction(self):
    self.outputfile.write( 'transitions \n' )
    self.outputfile.write( 'begin \n' )
    self.generateMessageClauses('if Alice.SendMsg', '[] Alice.SendMsg', 'sndmsg', self.numMsg)
    self.outputfile.write(';\n\n')
    self.generateMessageClauses('if Bob.SendAck', '[] Bob.SendAck', 'sndack', self.numAck)
    self.outputfile.write(';\n\n')
    self.generateChannelMsgs('sndmsg', 'rcvmsg', 'msgReceived', self.numMsg)
    self.outputfile.write(';\n\n')
    self.generateChannelMsgs('sndack', 'rcvack', 'ackReceived', self.numAck) 
    self.outputfile.write(';\n\n')
    self.generateMessageClauses('if msgReceived', '[] msgReceived', 'rcvmsg', self.numMsg)
    self.outputfile.write(';\n\n')
    self.generateMessageClauses('if ackReceived', '[] ackReceived', 'rcvack', self.numAck)
    self.outputfile.write('\n\n')
    self.outputfile.write( 'end\n\n')  

