#! /usr/bin/env python

# factory method to generate spec

class bgSpecification:
  def __init__(self, fileToWrite, numAgents):
    self.numAgents = numAgents    
    self.fileToWrite = fileToWrite

  def generateSpec(self):
    pass

  def generateAtomElem(self, atomStr): 
    elems = []   
    for i in range(self.numAgents):
      elems.append(('(' + atomStr + '[%d]' + ')') % i)
    return elems

  def generateDiffNumClause(self):
    oddn = self.numAgents    
    if self.numAgents % 2 == 0:
      oddn -= 1
    diffnums = []
    for i in range(1, (oddn+1), 2):
      diffnums.append( 'diffnum == %d' % i )    
    diffnumformula = ''
    for diffnum in diffnums[:(len(diffnums)-1)]:
      diffnumformula += diffnum + ' \\/ '
    diffnumformula += diffnums[len(diffnums)-1]
    return diffnumformula
                       
class fstBGSpecGenerator(bgSpecification):
  def __init__(self, fileToWrite, numAgents, runs):
    bgSpecification.__init__(self, fileToWrite, numAgents)
    self.runs = runs 

  def generateSpec(self):    
    self.fileToWrite.write( '\nspec_obs = AG (sndmsg[0] =>  Knows Alice Knows Bob sndmsg[0])\n\n' )
  
class sndBGSpecGenerator(bgSpecification):
  def __init__(self, fileToWrite, numAgents, runs):
    bgSpecification.__init__(self, fileToWrite, numAgents)
    self.runs = runs 

  def generateSpec(self):
    if self.numAgents % 2 == 0:
      numKnows = (self.numAgents-2) / 2
      numMsgs = self.numAgents/2 - 1
      self.fileToWrite.write( '\nspec_obs = AG (rcvmsg[%d] /\\ neg rcvack[%d] => \n' % (numMsgs, numMsgs) )
      knows = ''
      for i in range(numKnows):
        knows += ' Knows Alice Knows Bob '
      knows += ' Knows Alice rcvmsg[0] '
      self.fileToWrite.write( '                   ' )
      self.fileToWrite.write( '(' + knows + '))\n\n' )
    else:
      numKnows = (self.numAgents-1) / 2
      numMsgs = (self.numAgents-1) / 2
      numAcks = (self.numAgents-3) / 2
      self.fileToWrite.write( '\nspec_obs = AG (rcvack[%d] /\\ neg rcvmsg[%d] => \n' % (numAcks, numMsgs) )
      knows = ''
      for i in range(numKnows):
        knows += ' Knows Bob Knows Alice '
      knows += ' rcvmsg[0]'
      self.fileToWrite.write( '                   ' )
      self.fileToWrite.write( '(' + knows + '))\n\n' )
  
class trdBGSpecGenerator(bgSpecification):
  def __init__(self, fileToWrite, numAgents, runs):
    bgSpecification.__init__(self, fileToWrite, numAgents)
    self.runs = runs

  def generateSpec(self):    
    self.fileToWrite.write( '\nspec_obs = AF ((Knows Bob sndmsg[0]) \/ (Knows Alice rcvmsg[0]))\n\n' )
    
class fthBGSpecGenerator(bgSpecification):
  def __init__(self, fileToWrite, numAgents, runs):
    bgSpecification.__init__(self, fileToWrite, numAgents)
    self.runs = runs
  
  def generateSpec(self):
    if self.numAgents % 2 == 0:
      numKnows = self.numAgents / 2
      numAcks = self.numAgents/2 - 1
      self.fileToWrite.write( '\nspec_obs = AG (rcvack[%d] =>\n ' % (numAcks) )
      knows = ''
      for i in range(numKnows):
        knows += ' Knows Alice Knows Bob '
      knows += ' sndmsg[0]'
      self.fileToWrite.write( '                   ' )
      self.fileToWrite.write( '(' + knows + '))\n\n' )
    else:
      numKnows = (self.numAgents-1) / 2
      numMsgs = (self.numAgents-1) / 2      
      self.fileToWrite.write( '\nspec_obs = AG (rcvmsg[%d] => \n' % (numMsgs) )
      knows = ''
      for i in range(numKnows):
        knows += ' Knows Bob Knows Alice '
      knows += ' Knows Bob sndmsg[0] '
      self.fileToWrite.write( '                   ' )
      self.fileToWrite.write( '(' + knows + '))\n\n' )
      
