#! /usr/bin/env python

class dc_generator:
  def __init__(self, numAgents, outputfile):
    self.numAgents = numAgents
    self.outputfile = outputfile    

  def formulaComposition(self, head, tails, isAnd):
    lenTails = len(tails)    
    if lenTails == 0:
      return '(' + head + ')'
    elif lenTails == 1:
      if isAnd:
        #print '(' + str(head) + '/\\' + str(tails) + ')'
        return '(' + head + '/\\' + tails[0] + ')'
      else:
        return '(' + head + '\\/' + tails[0] + ')'
    else:
      fsttail = tails[0]
      if isAnd:
       newhead = '(' + head + '/\\' + fsttail + ')'
      else:
       newhead = '(' + head + '\\/' + fsttail + ')'            
      return self.formulaComposition(newhead, tails[1:], isAnd)     

  def defDataType(self):
    'define the data type for dc protocol'
    self.outputfile.write( 'type Num = {0..%d} \n' % self.numAgents )
    self.outputfile.write( 'paid : Bool[%d] \n' % self.numAgents )
    self.outputfile.write( 'chan : Bool[%d] \n' % self.numAgents )
    self.outputfile.write( 'said : Bool[%d] \n' % self.numAgents )
    self.outputfile.write( 'stop: Bool \n' )
    self.outputfile.write( 'diffnum: Num \n' )
    self.outputfile.write( '\n\n' )

  def generatePaidElem(self, index):
    elems = []
    if index == 0:
      for i in range(self.numAgents): 
         elems.append('(neg paid[%d])' % i)
      return elems
    else:
      for i in range(self.numAgents):
         if i == (index-1):
            elems.append('(paid[%d])' % i)
         else:
            elems.append('(neg paid[%d])' % i)
      return elems

  def defInitCond(self):
    'define the initial condition'
    # generate paid[0] ~ paid[n-1]
    paids = []
    for i in range(self.numAgents+1):
      elems = self.generatePaidElem(i)
      elemslen = len(elems)
      if elemslen == 1:
         head = elems[0]
         tails = []
      else:
         head = elems[0]
         tails = elems[1:]       
      cond = self.formulaComposition(head, tails, True)            
      paids.append(cond)
    
    partlyInitCond = self.formulaComposition(paids[0], paids[1:], False)  
    self.outputfile.write( 'init_cond = \n')  
    self.outputfile.write( str(partlyInitCond) + ' /\\ ( neg stop /\\ diffnum == 0) ' )
    self.outputfile.write( '\n\n' )

  def defAgents(self):
    for i in range(self.numAgents):
      agentStr = 'agent C%d "dc_agent_protocol_original" (paid[%d], ' % (i, i)
      if i == 0:
       chanVarsStr = ' chan[%d], chan[0], said, stop)' % (self.numAgents - 1)
      else:
       chanVarsStr = ' chan[%d], chan[%d], said, stop)' % (i-1, i)
      self.outputfile.write( agentStr + chanVarsStr + '\n' )
    self.outputfile.write( '\n\n' )      

  def generatePaidClause(self, index):
    elems = []
    if index == 0:
      for i in range(self.numAgents):
        elems.append('paid[%d] := False' % i)
    else:
      for i in range(self.numAgents):
        if i == (index-1):
          elems.append('paid[%d] := True' % i)
        else:
          elems.append('paid[%d] := False' % i)
    elemslen = len(elems)
    if elemslen == 1:
      return elemslen[0]
    else:
      allclause = ''
      for clause in elems[:(elemslen-1)]:
        allclause += clause + '; '
      allclause += elems[elemslen-1]
    return allclause

  def defTransaction(self):
    self.outputfile.write( 'transitions \n' )
    self.outputfile.write( 'begin \n' )
    self.outputfile.write( '   if neg stop -> \n' )
    self.outputfile.write( '     begin \n' )    
    # generate if clauses
    for i in range(self.numAgents):
      ifclause = \
       ('if C%d.SayDifferent -> begin said[%d]:=True; diffnum := diffnum + 1 end \n'+ 
        '       ' + '[] C%d.SaySame -> said[%d]:=False\n'+ 
        '       ' + 'fi;\n') % (i, i, i, i)
      ifclause = '       ' + ifclause
      self.outputfile.write(ifclause)
    self.outputfile.write( '\n' )
    self.outputfile.write( '     stop := True\n' )
    self.outputfile.write( '     end \n' )
    self.outputfile.write( '   [] stop ->\n' )     
    self.outputfile.write( '     begin \n' )    
    self.outputfile.write( '       stop := False;\n' )
    self.outputfile.write( '       diffnum := 0;\n' )
    self.outputfile.write( '       if ' )
    for i in range(self.numAgents+1):
      ifclause = \
        'True -> begin ' + self.generatePaidClause(i) + ' end\n'
      if i != 0:
        ifclause = '       [] ' + ifclause
      self.outputfile.write(ifclause)    
    self.outputfile.write( '       fi \n ' )
    self.outputfile.write( '    end\n' )
    self.outputfile.write( '  fi\n' )
    self.outputfile.write( 'end\n')

