#! /usr/bin/env python

# factory method to generate spec
# 1..4

class dcSpecification:
  def __init__(self, fileToWrite, numAgents):
    self.numAgents = numAgents    
    self.fileToWrite = fileToWrite

  def generateSpec(self):
    pass

  def generateAtomElem(self, atomStr): 
    elems = []   
    for i in range(self.numAgents):
      elems.append(('(' + atomStr + '[%d]' + ')') % i)
    return elems

  def generateDiffNumClause(self):
    oddn = self.numAgents    
    if self.numAgents % 2 == 0:
      oddn -= 1
    diffnums = []
    for i in range(1, (oddn+1), 2):
      diffnums.append( 'diffnum == %d' % i )    
    diffnumformula = ''
    for diffnum in diffnums[:(len(diffnums)-1)]:
      diffnumformula += diffnum + ' \\/ '
    diffnumformula += diffnums[len(diffnums)-1]
    return diffnumformula
                       
class fstSpecGenerator(dcSpecification):
  def __init__(self, fileToWrite, numAgents):
    dcSpecification.__init__(self, fileToWrite, numAgents)
    
  def generateSpec(self):
    paids = self.generateAtomElem('paid')
    self.fileToWrite.write( '\nspec_obs = AG ((stop /\ neg paid[0]) => \n' )
    self.fileToWrite.write( '                      ' )
    self.fileToWrite.write( '(Knows C0 ( ' )
    paidformula = ''
    for i in range(1, (self.numAgents-1)):
      paidformula += paids[i] + ' \\/ '
    paidformula += paids[self.numAgents-1]
    self.fileToWrite.write( paidformula + ' )))\n' )
  
class sndSpecGenerator(dcSpecification):
  def __init__(self, fileToWrite, numAgents):
    dcSpecification.__init__(self, fileToWrite, numAgents)    

  def generateSpec(self):
    knowPaids = self.generateAtomElem('Knows C0 paid')
    self.fileToWrite.write( '\nspec_obs = AG ((stop /\ neg paid[0] /\\ ( ' )
    diffnumClause = self.generateDiffNumClause()
    self.fileToWrite.write( diffnumClause )
    self.fileToWrite.write( ' )) => \n ' )    
    knowpaidformula = ''
    for i in range(1, (self.numAgents-1)):
      knowpaidformula += knowPaids[i] + ' \\/ '
    knowpaidformula += knowPaids[self.numAgents-1]
    self.fileToWrite.write( '(' + knowpaidformula + ')' + ' )\n' )
  
class trdSpecGenerator(dcSpecification):
  def __init__(self, fileToWrite, numAgents):
    dcSpecification.__init__(self, fileToWrite, numAgents)
    
  def generateSpec(self):
    paids = self.generateAtomElem('paid')
    self.fileToWrite.write( '\nspec_obs = AF ((neg paid[0] => \n' )
    self.fileToWrite.write( '                      ' )
    self.fileToWrite.write( 'Knows C0 ( ' )
    paidformula = ''
    for i in range(1, (self.numAgents-1)):
      paidformula += paids[i] + ' \\/ '
    paidformula += paids[self.numAgents-1]
    self.fileToWrite.write( paidformula + ' )))\n' )
  
class fthSpecGenerator(dcSpecification):
  def __init__(self, fileToWrite, numAgents):
    dcSpecification.__init__(self, fileToWrite, numAgents)
    
  def generateSpec(self):
    knowPaids = self.generateAtomElem('paid')
    self.fileToWrite.write( '\nspec_obs = AG (((stop /\ neg paid[0]) /\\ ( ' )
    diffnumClause = self.generateDiffNumClause()
    self.fileToWrite.write( diffnumClause )
    self.fileToWrite.write( ' )) => \n ' ) 
    self.fileToWrite.write( '                      ' )
    self.fileToWrite.write( '(Knows C0 (' )   
    knowpaidformula = ''
    for i in range(1, (self.numAgents-1)):
      knowpaidformula += knowPaids[i] + ' \\/ '
    knowpaidformula += knowPaids[self.numAgents-1]
    self.fileToWrite.write( knowpaidformula + ' )))\n' )

