#! /usr/bin/env python

class pe_generator:
  def __init__(self, numAgents, outputfile):
    self.numAgents = numAgents
    self.outputfile = outputfile    

  def formulaComposition(self, head, tails, isAnd):
    lenTails = len(tails)    
    if lenTails == 0:
      return '(' + head + ')'
    elif lenTails == 1:
      if isAnd:
        #print '(' + str(head) + '/\\' + str(tails) + ')'
        return '(' + head + '/\\' + tails[0] + ')'
      else:
        return '(' + head + '\\/' + tails[0] + ')'
    else:
      fsttail = tails[0]
      if isAnd:
       newhead = '(' + head + '/\\' + fsttail + ')'
      else:
       newhead = '(' + head + '\\/' + fsttail + ')'            
      return self.formulaComposition(newhead, tails[1:], isAnd)     

  def defDataType(self):
    'define the data type for dc protocol'
    self.outputfile.write( '''\
type Pos = {0..%d}\n
type Direct = {0..1}\n

-- Actual position of the evader
evaderPos : Pos\n
-- Position of the evader at the preceding moment of time
preEvaderPos : Pos\n

-- Position of the pursuer 
pursuerPos : Pos\n
-- Position of the pursuer  at the preceding moment of time
prePursuerPos : Pos\n

-- Current direction of pursuer .
direction: Direct\n  -- 0-forward 1-backward


-- This variable is initialised to false 
-- It is flagged the moment the pursuer receives a positive observation. 
found  : Bool\n''' % self.numAgents) 

  def defInitCond(self):
    'define the initial condition'
    self.outputfile.write( '''\
init_cond = pursuerPos == 0 /\ prePursuerPos == 0 /\ direction == 0 /\ \n
            evaderPos == %d /\ preEvaderPos == %d /\ \n
            neg found ''' % (self.numAgents, self.numAgents))
    self.outputfile.write( '\n\n' )

  def defAgents(self):
    self.outputfile.write('''\
agent Bob "Evader" (evaderPos, found)
agent Alice "Pursuer " (pursuerPos, direction, found)
    ''' )
    self.outputfile.write( '\n\n' )       

  def defTransaction(self):    
    self.outputfile.write('''\
transitions
begin

	prePursuerPos := pursuerPos;

	preEvaderPos := evaderPos;

	-- The pursuer  makes a move
	if Alice.ForwardOne	-> pursuerPos := pursuerPos + 1 
	[] Alice.BackwardOne	-> pursuerPos := pursuerPos - 1
	fi;
 
        if pursuerPos == 0 -> direction := 0
        [] pursuerPos == %d -> direction := 1
        fi;

	-- The evader makes a move
	if Bob.ForwardOne	-> evaderPos := evaderPos + 1 
	[] Bob.BackwardOne	-> evaderPos := evaderPos - 1
        [] Bob.Rests 		-> evaderPos := evaderPos
	fi;


	-- Has the pursuer  found the evader?

	if (pursuerPos == evaderPos ) \/
           (pursuerPos == preEvaderPos /\ evaderPos == prePursuerPos ) 
          -> found := True 
        [] otherwise -> found := False
        fi 

end''' % self.numAgents)
    self.outputfile.write('\n')
